function [STATUS, var_descriptions] = parse_STATUS(STATUS_folder)

files = dir(fullfile(STATUS_folder, '*.STATUS'));

%% Set up preallocation
% Get total number of points for the month. One file per day, one
% status report per hour
est_npoints = numel(files)*24;
% Preallocate vars with appropriate datatypes
[SR, Ns, acqInt, Ttx, Ntx, t_boot, status, acqDur] = ...
    deal(zeros(1, est_npoints, 'int32'));
[time, lat, lon, WTMP, Vbat, Isolar, magVar] = ...
    deal(zeros(1, est_npoints, 'double'));
[SN, Gpuc_ver, PC_ver, mode] = deal(cell(1, est_npoints));
var_descriptions = {...
    'time', 'MATLAB datenum'; ...
    'lat', 'Decimal Degrees N'; ...
    'lon', 'Decimal Degrees E'; ...
    'WTMP', 'Water Temperature (C)'; ...
    'Vbat', 'Battery Voltage'; ...
    'SR', 'Sampling Rate'; ...
    'Ns', 'Samples per channel'; ...
    'acqInt', 'Acquisition Interval (minutes)'; ...
    'Ttx', 'Transmission Interval (minutes)'; ...
    'Ntx', 'Number of Transmissions'; ...
    'SN', 'TRIAXYS Sensor S/N'; ...
    'Gpuc_ver', 'Gpuc EPROM Version'; ...
    'PC_ver', 'PC104 EPROM Version'; ...
    't_boot', 'Boot Time'; ...
    'status', 'Status Code'; ...
    'Isolar', 'Solar Current'; ...
    'magVar', 'Magnetic Variation'; ...
    'acqDur', 'Acquisition Duration (minutes)'; ...
    'mode', 'Mode'};
count = 0;
warning('off', 'MATLAB:table:ModifiedVarnames');
for i = 1:numel(files)
    fpath = fullfile(STATUS_folder, files(i).name);
    T = readtable(fpath, 'FileType', 'text', 'Delimiter', '\t', ...
        'Format', '%s %d %s %s %f %f %d %d %d %d %d %s %s %s %d %d %f %f %d %s');
    times = T.StatusMessageTime;
    years = floor(times/10^8) + 2000;
    months = floor(mod(times, 10^8)/10^6);
    days = floor(mod(times, 10^6)/10^4);
    hours = floor(mod(times, 10^4)/10^2);
    minutes = mod(times, 10^2);
    seconds = zeros(size(times));
    times = datenum(double([...
        years, months, days, hours, minutes, seconds]));
    if ~iscell(T.Location)  % All NaNs
        lats = NaN(size(T.Location));
        lons = NaN(size(T.Location));
    else
        [lats, lons] = cellfun(@parse_location_str, T.Location);
    end
    
    % insert data into vectors
    len = numel(times);
    idxs = count + 1:count + numel(times);
    time(idxs) = times;
    lat(idxs) = lats;
    lon(idxs) = lons;
    WTMP(idxs) = T.WaterTemperature;
    Vbat(idxs) = T.BatteryVoltage;
    SR(idxs) = T.SampleRate;
    Ns(idxs) = T.SamplesPerChannel;
    acqInt(idxs) = T.AcquisitionInterval_minutes_;
    Ttx(idxs) = T.TransmissionInterval_minutes_;
    Ntx(idxs) = T.NumberOfTransmissions;
    SN(idxs) = T.TRIAXYSSensorS_N;
    Gpuc_ver(idxs) = T.GpucEPROMVersion;
    PC_ver(idxs) = T.PC104EPROMVersion;
    t_boot(idxs) = T.BootTime;
    status(idxs) = T.StatusCode;
    Isolar(idxs) = T.SolarCurrent;
    magVar(idxs) = T.MagneticVariation;
    acqDur(idxs) = T.AcquisitionDuration_minutes_;
    mode(idxs) = T.Mode;
    count = count + len;
end
warning('on', 'MATLAB:table:ModifiedVarnames');

% Clear any extra entries from vectors
time(count+1:end) = [];
lat(count+1:end) = [];
lon(count+1:end) = [];
WTMP(count+1:end) = [];
Vbat(count+1:end) = [];
SR(count+1:end) = [];
Ns(count+1:end) = [];
acqInt(count+1:end) = [];
Ttx(count+1:end) = [];
Ntx(count+1:end) = [];
SN(count+1:end) = [];
Gpuc_ver(count+1:end) = [];
PC_ver(count+1:end) = [];
t_boot(count+1:end) = [];
status(count+1:end) = [];
Isolar(count+1:end) = [];
magVar(count+1:end) = [];
acqDur(count+1:end) = [];
mode(count+1:end) = [];

%%% Generate the output struct from the field names
field_names = var_descriptions(:, 1);
for i = 1:numel(field_names)
    fn = field_names{i};
    STATUS.(fn) = eval(fn);
end

end  % END FUNCTION parse_WAVE


function [lat, lon] = parse_location_str(location_str)
if ~ischar(location_str)
    error('Input #1 must be a string.');
end
result = regexp(location_str, '(\d+[.]\d+[NS])(\d+[.]\d+[EW])', 'tokens');
if isempty(result)
    lat = NaN;
    lon = NaN;
    return
end
a = result{1}{1};
b = result{1}{2};
lat = deg_min_to_dd(str2double(a(1:end-1))) * NSEW_sign(a(end));
lon = deg_min_to_dd(str2double(b(1:end-1))) * NSEW_sign(b(end));
end % End function parse_locations


function sign = NSEW_sign(NSEW)
if ~(isscalar(NSEW) && ischar(NSEW) ...
        && ismember(upper(NSEW), {'N', 'S', 'E', 'W'}))
    error('The argument must be ''N'', ''S'', ''E'', or ''W''');
end
if NSEW == 'S' || NSEW == 'W'
    sign = -1;
else
    sign = 1;
end
end % End function NSEW_sign


function dd = deg_min_to_dd(deg_min)
deg = floor(deg_min/100);
min = deg_min - deg*100;
dd = deg + min/60;
end % End function deg_min_to_dd
