function [WAVE, var_descriptions] = parse_WAVE(WAVE_folder)

files = dir(fullfile(WAVE_folder, '*.WAVE'));

%% Set up preallocation
% Get total number of points for the month. One file per day, one
% status report per hour
est_npoints = numel(files)*24;
% Preallocate vars with appropriate datatypes
[time, lat, lon] = deal(zeros(1, est_npoints, 'double'));
ZC = zeros(1, est_npoints, 'int16');
[Havg, Hmax, Hs, Tz, Ts, H10, T10, Tavg, Tp, Tp5, Hm0, ...
    MeanDirMag, MeanDirSpread] = deal(zeros(1, est_npoints, 'single'));
var_descriptions = {...
    'time', 'MATLAB datenum'; ...
    'lat', 'Decimal Degrees N'; ...
    'lon', 'Decimal Degrees E'; ...
    'ZC', 'Number of zero crossings'; ...
    'Havg', 'Average Wave Height'; ...
    'Hmax', 'Maximum Wave Height'; ...
    'Hs', 'Significant Wave Height'; ...
    'Tz', 'Zero Crossing Wave Period'; ...
    'Ts', 'Significant Wave Period'; ...
    'H10', 'Average Top 10% Wave Height'; ...
    'T10', 'Average Top 10% Wave Period'; ...
    'Tavg', 'Average Wave Period'; ...
    'Tp', 'Peak Wave Period'; ...
    'Tp5', 'Peak Wave Period by READ Method'; ...
    'Hm0', 'Significant Wave Height (sqrt method)'; ...
    'MeanDirMag', 'Mean Wave Direction (degrees magnetic)'; ...
    'MeanDirSpread', 'Wave Direction Spread (degrees)'};
count = 0;
warning('off', 'MATLAB:table:ModifiedVarnames');
for i = 1:numel(files)
    fpath = fullfile(WAVE_folder, files(i).name);
    T = readtable(fpath, 'FileType', 'text', 'Delimiter', '\t');
    years = T.Year;
    months = floor(T.MonthDay/100);
    days = mod(T.MonthDay, 100);
    hours = floor(T.Time/10000);
    minutes = floor(mod(T.Time, 10000)/100);
    seconds = mod(T.Time, 100);
    times = datenum([years, months, days, hours, minutes, seconds]);
    if ~iscell(T.Location)  % All NaNs
        lats = NaN(size(T.Location));
        lons = NaN(size(T.Location));
    else
        [lats, lons] = cellfun(@parse_location_str, T.Location);
    end
    
    % insert data into vectors
    len = numel(times);
    idxs = count + 1:count + numel(times);
    time(idxs) = times;
    lat(idxs) = lats;
    lon(idxs) = lons;
    ZC(idxs) = T.NumberOfZeroCrossings;
    Havg(idxs) = T.AverageWaveHeight_Havg_;
    Hmax(idxs) = T.MaxWaveHeight_Hmax_;
    Hs(idxs) = T.SignificantWaveHeight_Hsig_;
    Tz(idxs) = T.Tz;
    Ts(idxs) = T.SignificantWavePeriod_Tsig_;
    H10(idxs) = T.H10;
    T10(idxs) = T.T10;
    Tavg(idxs) = T.MeanPeriod;
    Tp(idxs) = T.PeakPeriod;
    Tp5(idxs) = T.Tp5;
    Hm0(idxs) = T.Hm0;
    MeanDirMag(idxs) = T.MeanMagneticDirection;
    MeanDirSpread(idxs) = T.MeanSpread;
    count = count + len;
end
warning('on', 'MATLAB:table:ModifiedVarnames');

% Clear any extra entries from vectors
time(count + 1:end) = [];
lat(count + 1:end) = [];
lon(count + 1:end) = [];
ZC(count + 1:end) = [];
Havg(count + 1:end) = [];
Hmax(count + 1:end) = [];
Hs(count + 1:end) = [];
Tz(count + 1:end) = [];
Ts(count + 1:end) = [];
H10(count + 1:end) = [];
T10(count + 1:end) = [];
Tavg(count + 1:end) = [];
Tp(count + 1:end) = [];
Tp5(count + 1:end) = [];
Hm0(count + 1:end) = [];
MeanDirMag(count + 1:end) = [];
MeanDirSpread(count + 1:end) = [];

%%% Generate the output struct from the field names
field_names = var_descriptions(:, 1);
for i = 1:numel(field_names)
    fn = field_names{i};
    WAVE.(fn) = eval(fn);
end

end  % END FUNCTION parse_WAVE


function [lat, lon] = parse_location_str(location_str)
if ~ischar(location_str)
    error('Input #1 must be a string.');
end
result = regexp(location_str, '(\d+[.]\d+[NS])(\d+[.]\d+[EW])', 'tokens');
if isempty(result)
    lat = NaN;
    lon = NaN;
    return
end
a = result{1}{1};
b = result{1}{2};
lat = deg_min_to_dd(str2double(a(1:end-1))) * NSEW_sign(a(end));
lon = deg_min_to_dd(str2double(b(1:end-1))) * NSEW_sign(b(end));
end % End function parse_locations


function sign = NSEW_sign(NSEW)
if ~(isscalar(NSEW) && ischar(NSEW) ...
        && ismember(upper(NSEW), {'N', 'S', 'E', 'W'}))
    error('The argument must be ''N'', ''S'', ''E'', or ''W''');
end
if NSEW == 'S' || NSEW == 'W'
    sign = -1;
else
    sign = 1;
end
end % End function NSEW_sign


function dd = deg_min_to_dd(deg_min)
deg = floor(deg_min/100);
min = deg_min - deg*100;
dd = deg + min/60;
end % End function deg_min_to_dd
